@@ -1,10 +1,10 @@
 package Cassandra::Client;
 
-use 5.008;
+use 5.010;
 use strict;
 use warnings;
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 use Cassandra::Client::Config;
 use Cassandra::Client::Connection;
@@ -13,6 +13,7 @@ use Cassandra::Client::AsyncIO;
 use Cassandra::Client::AsyncAnyEvent;
 use Cassandra::Client::Metadata;
 use Cassandra::Client::Pool;
+use Cassandra::Client::AdaptiveThrottler;
 use List::Util qw/shuffle/;
 use Promises qw/deferred/;
 use Clone qw/clone/;
@@ -47,6 +48,12 @@ sub new {
     $self->{async_io}= $async_io;
     $self->{metadata}= $metadata;
     $self->{pool}= $pool;
+    if ($options->{throttler}) {
+        my $throttler_class= "Cassandra::Client::$options->{throttler}";
+        $options->{throttler}= $throttler_class->new(%{$options->{throttler_config}});
+    } else {
+        $self->{throttler}= undef;
+    }
 
     return $self;
 }
@@ -176,128 +183,64 @@ sub _prepare {
         return _cb($callback);
     }
 
-    # XXX: Do we really need the duplication of having a second fast path for prepare()?
-    goto SLOWPATH if !$self->{connected};
-
-    my $connection= $self->{pool}->get_one;
-    goto SLOWPATH if !$connection;
-
-    $connection->prepare(sub {
-        my ($error)= @_;
-        if ($error && ref($error) && $error->retryable) {
-            return $self->_prepare_slowpath($callback, $query);
-        } else {
-            return _cb($callback, $error);
-        }
-    }, $query);
-
-    return;
-
-SLOWPATH:
-    return $self->_prepare_slowpath($callback, $query);
-}
-
-sub _prepare_slowpath {
-    my ($self, $callback, $query)= @_;
-
-    # XXX Do we need to retry prepares on other hosts if they fail for reasons other than bad CQL?
-
-    series([
-        sub {
-            my ($next)= @_;
-            $self->_connect($next);
-        }, sub {
-            my ($next)= @_;
-            $self->{pool}->get_one_cb($next);
-        }, sub {
-            my ($next, $connection)= @_;
-            $connection->prepare($next);
-        }
-    ], sub {
-        my ($error)= @_;
-        return _cb($callback, $error);
-    });
+    $self->_command("prepare", [ $callback, $query ]);
     return;
 }
 
 sub _execute {
     my ($self, $callback, $query, $params, $attribs)= @_;
-
-    my $params_copy= $params ? clone($params) : undef;
-    my $attribs_copy= $attribs ? clone($attribs) : undef;
-
-    goto SLOWPATH if !$self->{connected};
-
-    my $connection= $self->{pool}->get_one;
-    goto SLOWPATH if !$connection;
-
-    $connection->execute_prepared(sub {
-        my ($error, $result)= @_;
-        if ($error && ref($error) && $error->retryable) {
-            return $self->_execute_slowpath($callback, $query, $params_copy, $attribs_copy);
-        } else {
-            _cb($callback, $error, $result);
-        }
-        return;
-    }, \$query, $params_copy, $attribs_copy);
-
-    return;
-
-SLOWPATH:
-    return $self->_execute_slowpath($callback, $query, $params_copy, $attribs_copy);
-}
-
-sub _execute_slowpath {
-    my ($self, $callback, $query, $params_copy, $attribs_copy)= @_;
-
-    series([
-        sub {
-            my ($next)= @_;
-            $self->_connect($next);
-        }, sub {
-            my ($next)= @_;
-            $self->{pool}->get_one_cb($next);
-        }, sub {
-            my ($next, $connection)= @_;
-            $connection->execute_prepared($next, \$query, $params_copy, $attribs_copy);
-        }
-    ], sub {
-        my ($error, $result)= @_;
-        _cb($callback, $error, $result);
-        return;
-    });
+    $self->_command("execute_prepared", $callback, [ \$query, clone($params), clone($attribs) ]);
     return;
 }
 
 sub _batch {
     my ($self, $callback, $queries, $attribs)= @_;
+    $self->_command("execute_batch", $callback, [ clone($queries), clone($attribs) ]);
+    return;
+}
 
-    my $queries_copy= $queries ? clone($queries) : undef;
-    my $attribs_copy= $attribs ? clone($attribs) : undef;
+sub _wait_for_schema_agreement {
+    my ($self, $callback)= @_;
+    $self->_command("wait_for_schema_agreement", $callback, []);
+    return;
+}
+
+
+# Command queue
+sub _command {
+    my ($self, $command, $callback, $args)= @_;
+
+    # Handle overloads
+    goto FAILFAST if $self->{throttler} && $self->{throttler}->should_fail();
 
     goto SLOWPATH if !$self->{connected};
 
     my $connection= $self->{pool}->get_one;
     goto SLOWPATH if !$connection;
 
-    $connection->execute_batch(sub {
+    $connection->$command(sub {
         my ($error, $result)= @_;
-        if ($error && ref($error) && $error->retryable) {
-            return $self->_batch_slowpath($callback, $queries_copy, $attribs_copy);
-        } else {
-            _cb($callback, $error, $result);
+        if (my $throttler= $self->{throttler}) {
+            $throttler->count($error);
         }
-        return;
-    }, $queries_copy, $attribs_copy);
+
+        if ($error && ref($error) && $error->retryable) {
+            return $self->_command_slowpath($command, $callback, $args);
+        }
+        return _cb($callback, $error, $result);
+    }, @$args);
 
     return;
 
 SLOWPATH:
-    return $self->_batch_slowpath($callback, $queries_copy, $attribs_copy);
+    return $self->_command_slowpath($command, $callback, $args);
+
+FAILFAST:
+    return _cb($callback, "Client-induced failure by throttling mechanism");
 }
 
-sub _batch_slowpath {
-    my ($self, $callback, $queries_copy, $attribs_copy)= @_;
+sub _command_slowpath {
+    my ($self, $command, $callback, $args)= @_;
 
     series([
         sub {
@@ -308,12 +251,14 @@ sub _batch_slowpath {
             $self->{pool}->get_one_cb($next);
         }, sub {
             my ($next, $connection)= @_;
-            $connection->execute_batch($next, $queries_copy, $attribs_copy);
+            $connection->$command($next, @$args);
         }
     ], sub {
         my ($error, $result)= @_;
-        _cb($callback, $error, $result);
-        return;
+        if (my $throttler= $self->{throttler}) {
+            $throttler->count($error);
+        }
+        return _cb($callback, $error, $result);
     });
     return;
 }
@@ -348,20 +293,6 @@ sub _each_page {
     return;
 }
 
-sub _wait_for_schema_agreement {
-    my ($self, $callback)= @_;
-
-    # Let's face it, perf doesn't matter on this method :-)
-    $self->{pool}->get_one_cb(sub {
-        my ($error, $connection)= @_;
-        return _cb($callback, $error) if $error;
-
-        $connection->wait_for_schema_agreement(sub {
-            return _cb($callback, @_);
-        });
-    });
-}
-
 
 # Utility functions for callers
 sub _cb {
