@@ -1,5 +1,5 @@
 package Cassandra::Client::Config;
-use 5.008;
+use 5.010;
 use strict;
 use warnings;
 
@@ -19,6 +19,8 @@ sub new {
         timer_granularity   => 0.1,
         request_timeout     => 11,
         warmup              => 0,
+        throttler           => undef,
+        throttler_config    => undef,
     }, $class;
 
     if (my $cp= $config->{contact_points}) {
@@ -47,12 +49,19 @@ sub new {
     }
 
     # Strings
-    for (qw/cql_version keyspace compression default_consistency/) {
+    for (qw/cql_version keyspace compression default_consistency throttler/) {
         if (exists($config->{$_})) {
             $self->{$_}= defined($config->{$_}) ? "$config->{$_}" : undef;
         }
     }
 
+    # Arbitrary hashes
+    for (qw/throttler_config/) {
+        if (exists($config->{$_})) {
+            $self->{$_}= $config->{$_};
+        }
+    }
+
     $self->{username}= $config->{username};
     $self->{password}= $config->{password};
 
